#!/bin/sh
################################################################################
# run_override.sh
#
# Processes an override package believed to be present on a mounted USB stick.
#
# SVN Keywords
# $HeadURL: http://nanoscience.oxinst.com/svn/Mercury/target_files/MercuryTarget/opt/OxfordInstruments/cryosys/run_override.sh $
# $Author: robertsi $
# $Revision: 5439 $
# $Date: 2014-01-31 15:25:56 +0000 (Fri, 31 Jan 2014) $
#
# Copyright 2012 - Oxford Instruments
# This material is protected by copyright law. It is unlawful to copy it.
#
# This document contains confidential information. It is not to be disclosed or
# used except in accordance with applicable contracts or agreements. This
# document must be rendered illegible when being discarded.
#
################################################################################

# Exit on error.
set -e

source /opt/OxfordInstruments/cryosys/utils.sh

LOG_LEVEL=$LL_TRACE
LOG_MSG_PREFIX="run_override.sh"

OVERRIDE_PKG_WORKING_LOCN="/tmp/cryosys_override"
OVERRIDE_PKG_UNPACKED_LOCN="$OVERRIDE_PKG_WORKING_LOCN/unpacked"
OVERRIDE_IMAGE_LOCN="/etc"

OVERRIDE_PKG_NAME="cryosys.override"
OVERRIDE_ARCHIVE_NAME="cryosys_override.tar"
OVERRIDE_SCRIPT_NAME="cryosys_override.sh"

OVERRIDE_COPYING_IMAGE="$OVERRIDE_IMAGE_LOCN/override_pkg_copying.raw"
OVERRIDE_DECRYPTING_IMAGE="$OVERRIDE_IMAGE_LOCN/override_pkg_decrypting.raw"
OVERRIDE_EXTRACTING_IMAGE="$OVERRIDE_IMAGE_LOCN/override_pkg_extracting.raw"
OVERRIDE_EXECING_IMAGE="$OVERRIDE_IMAGE_LOCN/execing_override.raw"

OVERRIDE_SYS_ERR_IMAGE="$OVERRIDE_IMAGE_LOCN/override_pkg_system_err.raw"
OVERRIDE_UNAUTH_PKG_IMAGE="$OVERRIDE_IMAGE_LOCN/unauthorised_override_pkg.raw"
OVERRIDE_INVALID_PKG_IMAGE="$OVERRIDE_IMAGE_LOCN/invalid_override_pkg.raw"
OVERRIDE_INVALID_ARCHIVE_IMAGE="$OVERRIDE_IMAGE_LOCN/invalid_override_archive.raw"
OVERRIDE_PKG_INCOMPLETE_IMAGE="$OVERRIDE_IMAGE_LOCN/override_missing_file.raw"

################################################################################
# Function Definitions.
################################################################################

fnCopyPackage()
################################################################################
{
    fnShowImage "$OVERRIDE_COPYING_IMAGE"

    set +e
    mkdir -p "$OVERRIDE_PKG_WORKING_LOCN"
    cp "$USB_MOUNT_POINT/$OVERRIDE_PKG_NAME" "$OVERRIDE_PKG_WORKING_LOCN"
    local _RESULT=$?
    set -e

    if [ $_RESULT -ne 0 ]; then
        fnLog $LL_OUTCOME "Error: Copy of package to $OVERRIDE_PKG_WORKING_LOCN failed."
        fnShowImage "$OVERRIDE_SYS_ERR_IMAGE"
        exit 1
    fi
}

fnDecryptPackage()
################################################################################
{
    fnShowImage "$OVERRIDE_DECRYPTING_IMAGE"

    local _GOOD_VALIDATION_RESULT_CONTENT_1='using RSA key ID 8DD4034E '
    local _GOOD_VALIDATION_RESULT_CONTENT_2='gpg: Good signature from "My Mercury Support (Mercury Software Producer) <mymercurysupport@oxinst.com>"'

    local _REPORT=""
    set +e
    _REPORT=`HOME="/root" && cat "/root/.gnupg/pph.gpg" | \
             /usr/bin/gpg \
                 --batch \
                 --passphrase-fd 0 \
                 --ignore-time-conflict \
                 --output "$OVERRIDE_PKG_WORKING_LOCN/$OVERRIDE_ARCHIVE_NAME" \
                 --decrypt "$OVERRIDE_PKG_WORKING_LOCN/$OVERRIDE_PKG_NAME" \
                 2>&1`
    local _RESULT=$?
    set -e
    _REPORT=`echo "$_REPORT" | tr '\n' ' '`

    if [ $_RESULT -ne 0 ]; then
        fnLog $LL_OUTCOME "Error: gpg op failed. Exit code $_RESULT. Report: $_REPORT"

        # A not-very-rigorous token attempt at presenting the user with a
        # message relevant to the failure by parsing the error report.
        if [ "`echo "$_REPORT" | grep 'ignature'`" != "" ]; then
            fnLog $LL_TRACE "Interpreting error report as signature validation failure."
            fnShowImage "$OVERRIDE_UNAUTH_PKG_IMAGE"
        else
            fnLog $LL_TRACE "Interpreting error report as decryption failure."
            fnShowImage "$OVERRIDE_INVALID_PKG_IMAGE"
        fi
        exit 1
    fi

    if [ "`echo "$_REPORT" | \
           grep "$_GOOD_VALIDATION_RESULT_CONTENT_1" | \
           grep "$_GOOD_VALIDATION_RESULT_CONTENT_2"`" = "" ]; then
        fnLog $LL_OUTCOME "Error: Decryption succeeded but signature is wrong. Report: $_REPORT"
        fnShowImage "$OVERRIDE_UNAUTH_PKG_IMAGE"
        exit 1
    fi
}

fnExtractAndValidateArchive()
################################################################################
{
    fnShowImage "$OVERRIDE_EXTRACTING_IMAGE"

    set +e
    mkdir -p "$OVERRIDE_PKG_UNPACKED_LOCN"
    tar -xv -C "$OVERRIDE_PKG_UNPACKED_LOCN" -f "$OVERRIDE_PKG_WORKING_LOCN/$OVERRIDE_ARCHIVE_NAME"
    local _RESULT=$?
    set -e

    if [ $_RESULT -ne 0 ]; then
        fnLog $LL_OUTCOME "Error: Tarball extraction failed."
        fnShowImage "$OVERRIDE_INVALID_ARCHIVE_IMAGE"
        exit 1
    fi

    if [ ! -f "$OVERRIDE_PKG_UNPACKED_LOCN/$OVERRIDE_SCRIPT_NAME" ]; then
        fnLog $LL_OUTCOME "Error: $OVERRIDE_SCRIPT_NAME missing from override package."
        fnShowImage "$OVERRIDE_PKG_INCOMPLETE_IMAGE"
        exit 1
    fi
}

fnLaunchScript()
################################################################################
{
    fnShowImage "$OVERRIDE_EXECING_IMAGE"
    chmod a+x "$OVERRIDE_PKG_UNPACKED_LOCN/$OVERRIDE_SCRIPT_NAME"
    sh "$OVERRIDE_PKG_UNPACKED_LOCN/$OVERRIDE_SCRIPT_NAME" &
}

################################################################################
# Entry.
################################################################################

cd "$CRYOSYS_INSTALL_LOCN"

fnLog $LL_TRACE "Processing override package."

fnLog $LL_TRACE "Copying override package..."
fnCopyPackage
fnLog $LL_TRACE "Copied."

fnLog $LL_TRACE "Decrypting override package..."
fnDecryptPackage
fnLog $LL_TRACE "Decrypted."

fnLog $LL_TRACE "Extracting and validating tarball contents..."
fnExtractAndValidateArchive
fnLog $LL_TRACE "Extracted and validated."

fnLog $LL_PROGRESS "Launching override script."
fnLaunchScript

exit 0
